/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.sim.sim;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Applikationsklasse zum Erzeugen von Simulationsstrecken
 * 
 * @author Dambach Werke GmbH
 * @version $Revision: 1.1 $ / $Date: 2008/10/31 13:30:31 $ / ($Author: Sans $)
 */
public class NeueSimulationsStrecke implements Runnable, StandardApplication 
                                      
{
  /**
   * Debug-Logger für Logging-Ausgaben
   */
  private Debug _debug;

  /**
   * -konfigBereich von Kommandozeile
   */
  private String _pidKonfigBereich = null;

  /**
   * -konfigObjekt von Kommandozeile
   */
  private String _pidKonfigObjekt = null;

  /**
   * -pidPrefix von Kommandozeile
   */
  private String _pidPrefix = "simulationsStrecke";

  /**
   * _namePrefix von Kommandozeile
   */
  private String _namePrefix = "SimulationsStrecke";

  /**
   * Konfiurationsbereich
   */
  private ConfigurationArea _konfigBereich;

  /**
   * Konfigurationsobjekt
   */
  private ConfigurationObject _konfigObjekt;

  /**
   * Datenverteiler-Verbindung
   */
  private ClientDavInterface _connection;

  /**
   * Mengenbezeichner für die dynamische Menge
   */
  static final String MENGE = "SimulationsStrecken";

  /**
   * Default- Konstruktor.<br>
   * Wird typischerweise von <code>main</code> aufgerufen
   */
  public NeueSimulationsStrecke()
  {
    try
    {
      Thread.sleep(100);
    }
    catch (InterruptedException e)
    {
    }
  }

  /**
   * Überschriebene Methode von <code>StandardApplication</code>,
   * die die speziellen Startparameter auswertet
   *
   * @param argumentList
   *  siehe sys.funclib.application.StandardApplication#parseArguments(sys.funclib.ArgumentList)
   */
  public void parseArguments(ArgumentList argumentList)
  {
    _debug = Debug.getLogger();

    if (argumentList.hasArgument("-konfigBereich"))
    {
      _pidKonfigBereich = argumentList.fetchArgument("-konfigBereich=").asNonEmptyString();
    }

    if (argumentList.hasArgument("-konfigObjekt"))
    {
      _pidKonfigObjekt = argumentList.fetchArgument("-konfigObjekt=").asNonEmptyString();
    }

    if (argumentList.hasArgument("-pidPrefix"))
    {
      _pidPrefix = argumentList.fetchArgument("-pidPrefix=").asNonEmptyString();
    }

    if (argumentList.hasArgument("-namePrefix"))
    {
      _namePrefix = argumentList.fetchArgument("-namePrefix").asNonEmptyString();
    }

    argumentList.fetchUnusedArguments();
  }

  /**
   * Überschriebene Methode von StandardApplication, die die Initialisierung durchführt.<br>
   * Es wird eine Instanz von <code>VerwaltungSimulation</code> erzeugt und initialisiert
   * 
   * @param connection
   * siehe sys.funclib.application.StandardApplication#initialize(de.bsvrz.dav.daf.main.ClientDavInterface)
   */
  public void initialize(ClientDavInterface connection)
  {
    _connection = connection;
    
    if (_pidKonfigObjekt != null)
    {
      _konfigObjekt = (ConfigurationObject)connection.getDataModel().getObject(_pidKonfigObjekt); 
    }
    else
    {
      _konfigObjekt = connection.getLocalConfigurationAuthority();
    }
    
    if (_pidKonfigBereich != null)
    {
      _konfigBereich = connection.getDataModel().getConfigurationArea(_pidKonfigBereich);
    }
    else
    {
      ConfigurationObject co = connection.getDataModel().getConfigurationAuthority();
      AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.konfigurationsVerantwortlicherEigenschaften");
      Data data = co.getConfigurationData(atg);
      if (data != null)
      {
        TextArray array = data.getTextArray("defaultBereich");
        if (array.getLength() > 0)
        {
          _konfigBereich = (ConfigurationArea)connection.getDataModel().getObject(array.getText(0));
        }
      }
    }
    
    new Thread(this).start();
  }
  
  public void run()
  {
    short ext = 1;
    DynamicObject simulationsStrecke = null;
    DynamicObjectType typ = (DynamicObjectType)_connection.getDataModel().getType("typ.simulationsStrecke");
    
    while(ext < Short.MAX_VALUE && _connection.getDataModel().getObject(_pidPrefix + ext) != null)
    {
      ext += 1;
    }
    
    try
    {
      simulationsStrecke = _konfigBereich.createDynamicObject(typ, _pidPrefix + ext, _namePrefix + ext, null);
      _konfigObjekt.getMutableSet(MENGE).add(simulationsStrecke);
      _debug.info("Dynamisches Objekt \"" + _pidPrefix + ext + "\" vom Typ " + typ.getPid() + " erfolgreich erzeugt");
      System.exit(0);
    }
    catch(ConfigurationChangeException e)
    {
      if (simulationsStrecke != null)
      {
        try
        {
          simulationsStrecke.invalidate();
        }
        catch(ConfigurationChangeException ex)
        {
          // man kann nichts mehr tun
        }
      }
      _debug.error("Fehler bei Erzeugen eines dynamisches Objekts \"" + _pidPrefix + "\" vom Typ " + typ.getPid(), e);
      System.exit(1);
    }
  }
  
  /**
   * Programmeinstieg
   * @param arguments Parameter
   */
  public static void main(String[] arguments)
  {
    StandardApplicationRunner.run(new NeueSimulationsStrecke(), arguments);
  }
  
}
