/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
 
package de.bsvrz.vew.sim.sim.steuerung;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.TextValue;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.vew.sim.sim.datengenerator.DatenGeneratorOnlineSimulation;
import de.bsvrz.vew.sim.sim.datengenerator.DatenGeneratorSimulation;
import de.bsvrz.vew.sim.sim.simulation.OnlineSimulation;
import de.bsvrz.vew.sim.sim.simulation.Simulation;
import de.bsvrz.vew.sim.sim.simulationsstrecke.SimulationsStrecke;

/**
 * Steuerung zu einer Online-Simulation.
 *  
 * @version  $Revision: 1.4 $ / $Date: 2008/12/03 14:00:33 $ / ($Author: Sans $)
 * @author  Dambach-Werke GmbH
 * @author  Stefan Sans
 */
public class SteuerungOnlineSimulation extends SteuerungSimulationAbstrakt
{
  
  /**
   * Debug-Logger für Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();

  /**
   * Erzeuge eine neue Steuerungs-Instanz für eine Online-Simulation
   * @param simulation die Simulation, zu der die Instanz erzeugt werden soll
   */
  public SteuerungOnlineSimulation(Simulation simulation)
  {
    super(simulation);
  }

  @Override
  public boolean sendeZustand()
  {
    ClientDavInterface con = _simulation.getConnection();
    AttributeGroup atg = con.getDataModel().getAttributeGroup("atg.simulationsSteuerungOnline");
    Aspect asp = con.getDataModel().getAspect("asp.zustand");

    String zustand = _simulation.getZustand().toString();
    Data data = con.createData(atg);
    TextValue textValue = data.getTextValue("SimulationsZustand");
    textValue.setText(zustand);

    _debug.info("Publiziere Zustand \"" + zustand +
        "\" für Simulation " + _simulation);

    if (!_simulation.sendeAlsQuelle(atg, asp, data))
    {
      _debug.error("Fehler beim Senden an Simulationsobjekt " + _simulation);
     return false;
    }
    _debug.fine("Zustand Simulation " + _simulation + " erfolgreich versendet");
    return true;
  }

  @Override
  public boolean erstelleWorkflow(int uebergang)
  {
    switch(uebergang)
    {
      case 0: // Neu -> Vorstart
      {
        WarteThread t1 = new WarteThread()
        {
          public void run()
          {
            setResult(isSimulationsStreckenObjektGueltig());
          }
        };

        WarteThread t2 = new WarteThread();

        t1.start(new WarteThread[] { });
        t2.start(new WarteThread[] { t1 });

        return t2.getResult();
      }

      case 1: // Vorstart -> Start
      {
        WarteThread t1 = new WarteThread()
        {
          public void run()
          {
            setResult(isKonfigurationBereit());
          }
        };

        WarteThread t2 = new WarteThread()
        {
          public void run()
          {
            setResult(isParametrierungBereit());
          }
        };

        WarteThread t3 = new WarteThread()
        {
          public void run()
          {
            setResult(isArchivSystemBereit());
          }
        };

        WarteThread t4 = new WarteThread()
        {
          public void run()
          {
            setResult(sendeStart());
          }
        };

        WarteThread t5 = new WarteThread()
        {
          public void run()
          {
            starteDatengenerator();
          }
        };

        WarteThread t6 = new WarteThread();

        t1.start(new WarteThread[] { });
        t2.start(new WarteThread[] { });
        t3.start(new WarteThread[] { });
        t4.start(new WarteThread[] { t1, t2, t3 });
        t5.start(new WarteThread[] { t4 });
        t6.start(new WarteThread[] { t5 });

        return t6.getResult();
      }

      case 2: // Start -> Stop
      {
        WarteThread t1 = new WarteThread()
        {
          public void run()
          {
            beendeDatengenerator();
          }
        };

        WarteThread t2 = new WarteThread()
        {
          public void run()
          {
            sendeStop();
          }
        };

        WarteThread t3 = new WarteThread();

        t1.start(new WarteThread[] { });
        t2.start(new WarteThread[] { t1 });
        t3.start(new WarteThread[] { t2 });

        return t3.getResult();
      }

      case 3: // Stop -> gelöscht
      {
        WarteThread t1 = new WarteThread()
        {
          public void run()
          {
            setResult(sendeBeenden());
          }
        };

        WarteThread t2 = new WarteThread();

        t1.start(new WarteThread[] { });
        t2.start(new WarteThread[] { t1 });

        return t2.getResult();
      }

      case 4: // Stop -> Vorstart
      {
        WarteThread t1 = new WarteThread()
        {
          public void run()
          {
            setResult(sendeBeenden());
          }
        };

        WarteThread t2 = new WarteThread();

        t1.start(new WarteThread[] { });
        t2.start(new WarteThread[] { t1 });

        return (t2.getResult() && erstelleWorkflow(0));
      }
    }

    return false;

  }

  public boolean isSimulationsStreckenObjektGueltig()
  {
    int result = 0;
    SimulationsStrecke ss = _simulation.getSimulationsStrecke();

    _debug.fine("Prüfe Gültigkeit Simulationsstrecke " + ss);

    if (!(ss.getStartSkriptInfo().size() > 0))
    {
      result = 1;
    }

    if (result == 0)
    {
      _debug.info("Simulationsstrecke " + ss + " ist gültig");
    }
    else
    {
      if (result == 1)
      {
        _debug.error("Simulationsstrecke " + ss + " hat Startskriptliste der Länge 0");
      }
      _debug.error("Simulationsstrecke " + ss + " ist ungültig");
    }

    return (result == 0);
  }

  /**
   * Max. Zeit, die auf einen Wechsel zu einem erwarteten Status gewartet wird
   */
  public final static long STATUS_TIMEOUT = 20000;

  public boolean isKonfigurationBereit()
  {
    long aktZeit = System.currentTimeMillis();

    _debug.fine("Warte max. " + STATUS_TIMEOUT + "ms auf positiven Status Konfiguration für Simulation " +
        _simulation);

    do
    {
      if (_simulation.getStatusKonfiguration() == Simulation.KONFIG_STATUS_BEREIT)
      {
        _debug.info("Konfiguration ist bereit für Simulation " + _simulation);
        return true;
      }
      _simulation.getConnection().sleep(50);
    }
    while(System.currentTimeMillis() - aktZeit < STATUS_TIMEOUT);

    _debug.error("Konfiguration ist nicht bereit für Simulation " + _simulation);

    return false;
  }

  public boolean isParametrierungBereit()
  {
    long aktZeit = System.currentTimeMillis();

    _debug.fine("Warte max. " + STATUS_TIMEOUT + " auf positiven Status Paramtrierung für Simulation " +
        _simulation);

    do
    {
      if (_simulation.getStatusParametrierung() == Simulation.PARAM_STATUS_AKTIV)
      {
        _debug.info("Paramtrierung ist bereit für Simulation " + _simulation);
        return true;
      }
      _simulation.getConnection().sleep(50);
    }
    while(System.currentTimeMillis() - aktZeit < STATUS_TIMEOUT);

    _debug.error("Paramtrierung ist nicht bereit für Simulation " + _simulation);

    return false;
  }

  public boolean isArchivSystemBereit()
  {
    long aktZeit = System.currentTimeMillis();

    _debug.fine("Warte max. " + STATUS_TIMEOUT + " auf positiven Status Archiv für Simulation " +
        _simulation);

    do
    {
      if (_simulation.getStatusArchiv() == Simulation.ARC_STATUS_AKTIV)
      {
        _debug.info("Archiv ist bereit für Simulation " + _simulation);
        return true;
      }
      _simulation.getConnection().sleep(50);
    }
    while(System.currentTimeMillis() - aktZeit < STATUS_TIMEOUT);

    _debug.error("Archiv ist nicht bereit für Simulation " + _simulation);

    return false;
  }

  /**
   * Der Datengenerator für diese Simulation
   */
  protected DatenGeneratorSimulation _generator;

  /**
   * Starte den Datengenerator für diese Simulation
   */
  protected void starteDatengenerator()
  {
    _debug.info("Starte Datengenerator für Online-Simulation " + _simulation);

    if (_generator == null)
    {
      _generator = new DatenGeneratorOnlineSimulation((OnlineSimulation)_simulation) ;
    }

    _generator.start();
  }

  /**
   * Beende den Datengenerator für diese Simulation
   */
  protected void beendeDatengenerator()
  {
    _debug.info("Beende Datengenerator für Online-Simulation " + _simulation);
    
    _generator.stop();
    _generator = null;
  }
  
}
